LIBRARY liblouis

EXPORTS
	lou_getProgramPath
	lou_version
	lou_charSize
	lou_translateString
	lou_translate
	lou_hyphenate
	lou_dotsToChar
	lou_charToDots
	lou_backTranslateString
	lou_backTranslate
	lou_logPrint
	lou_logFile
	lou_logEnd
	lou_compileString
	lou_setDataPath
	lou_getTable
	lou_getDataPath
	lou_registerLogCallback
	lou_setLogLevel
	lou_free
	getDotsForChar
	getCharFromDots
	showString
	showDots
	showAttributes
	findOpcodeNumber
	findOpcodeName
	extParseChars
	extParseDots
	getLastTableList
lou_readCharFromFile
lou_registerTableResolver
lou_translatePrehyphenated
